@@ -85,7 +85,7 @@ static S2N_RESULT s2n_generate_client_session_id(struct s2n_connection *conn)
 
     /* QUIC should not allow session ids for any reason.
      *
-     *= https://tools.ietf.org/rfc/rfc9001#section-8.4
+     *= https://www.rfc-editor.org/rfc/rfc9001#section-8.4
      *# A server SHOULD treat the receipt of a TLS ClientHello with a non-empty
      *# legacy_session_id field as a connection error of type PROTOCOL_VIOLATION.
      */
@@ -225,7 +225,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
     }
 
     /*
-     *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+     *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
      *# The client will also send a
      *# ClientHello when the server has responded to its ClientHello with a
      *# HelloRetryRequest.  In that case, the client MUST send the same
@@ -292,7 +292,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
 
         switch (extension_type) {
             /*
-             *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+             *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
              *#    -  If a "key_share" extension was supplied in the HelloRetryRequest,
              *#       replacing the list of shares with a list containing a single
              *#       KeyShareEntry from the indicated group.
@@ -301,7 +301,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
                 /* Handled when parsing the key share extension */
                 break;
             /*
-             *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+             *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
              *#    -  Removing the "early_data" extension (Section 4.2.10) if one was
              *#       present.  Early data is not permitted after a HelloRetryRequest.
              */
@@ -309,7 +309,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
                 RESULT_ENSURE(new_size == 0, S2N_ERR_BAD_MESSAGE);
                 break;
             /*
-             *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+             *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
              *#    -  Including a "cookie" extension if one was provided in the
              *#       HelloRetryRequest.
              */
@@ -317,7 +317,7 @@ static S2N_RESULT s2n_client_hello_verify_for_retry(struct s2n_connection *conn,
                 /* Handled when parsing the cookie extension */
                 break;
             /*
-             *= https://tools.ietf.org/rfc/rfc8446#section-4.1.2
+             *= https://www.rfc-editor.org/rfc/rfc8446#section-4.1.2
              *#    -  Updating the "pre_shared_key" extension if present by recomputing
              *#       the "obfuscated_ticket_age" and binder values and (optionally)
              *#       removing any PSKs which are incompatible with the server's
@@ -481,7 +481,7 @@ int s2n_parse_client_hello(struct s2n_connection *conn)
      *     A TLS-compliant application MUST support key exchange with secp256r1 (NIST P-256)
      *     and SHOULD support key exchange with X25519 [RFC7748]
      *
-     *= https://tools.ietf.org/rfc/rfc4492#section-4
+     *= https://www.rfc-editor.org/rfc/rfc4492#section-4
      *# A client that proposes ECC cipher suites may choose not to include these extensions.
      *# In this case, the server is free to choose any one of the elliptic curves or point formats listed in Section 5.
      *
@@ -557,7 +557,7 @@ int s2n_process_client_hello(struct s2n_connection *conn)
      * Negotiate protocol version, cipher suite, ALPN, select a cert, etc. */
     struct s2n_client_hello *client_hello = &conn->client_hello;
 
-    const struct s2n_security_policy *security_policy;
+    const struct s2n_security_policy *security_policy = NULL;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
     if (!s2n_connection_supports_tls13(conn) || !s2n_security_policy_supports_tls13(security_policy)) {
@@ -599,7 +599,7 @@ int s2n_process_client_hello(struct s2n_connection *conn)
     /* Check if this is the second client hello in a hello retry handshake */
     if (s2n_is_hello_retry_handshake(conn) && conn->handshake.message_number > 0) {
         /**
-         *= https://tools.ietf.org/rfc/rfc8446#4.1.4
+         *= https://www.rfc-editor.org/rfc/rfc8446#4.1.4
          *# Servers MUST ensure that they negotiate the
          *# same cipher suite when receiving a conformant updated ClientHello (if
          *# the server selects the cipher suite as the first step in the
@@ -701,7 +701,7 @@ int s2n_client_hello_send(struct s2n_connection *conn)
 {
     POSIX_ENSURE_REF(conn);
 
-    const struct s2n_security_policy *security_policy;
+    const struct s2n_security_policy *security_policy = NULL;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
     const struct s2n_cipher_preferences *cipher_preferences = security_policy->cipher_preferences;
@@ -756,7 +756,7 @@ int s2n_client_hello_send(struct s2n_connection *conn)
 
     /**
      * For initial handshakes:
-     *= https://tools.ietf.org/rfc/rfc5746#3.4
+     *= https://www.rfc-editor.org/rfc/rfc5746#3.4
      *# o  The client MUST include either an empty "renegotiation_info"
      *#    extension, or the TLS_EMPTY_RENEGOTIATION_INFO_SCSV signaling
      *#    cipher suite value in the ClientHello.  Including both is NOT
@@ -765,7 +765,7 @@ int s2n_client_hello_send(struct s2n_connection *conn)
      * rather than the "renegotiation_info" extension.
      *
      * For renegotiation handshakes:
-     *= https://tools.ietf.org/rfc/rfc5746#3.5
+     *= https://www.rfc-editor.org/rfc/rfc5746#3.5
      *# The SCSV MUST NOT be included.
      */
     if (tls12_is_possible && !s2n_handshake_is_renegotiation(conn)) {
@@ -774,7 +774,10 @@ int s2n_client_hello_send(struct s2n_connection *conn)
     }
 
     /* Write size of the list of available ciphers */
-    POSIX_GUARD(s2n_stuffer_write_vector_size(&available_cipher_suites_size));
+    uint32_t ciphers_size = 0;
+    POSIX_GUARD(s2n_stuffer_get_vector_size(&available_cipher_suites_size, &ciphers_size));
+    POSIX_ENSURE(ciphers_size > 0, S2N_ERR_INVALID_CIPHER_PREFERENCES);
+    POSIX_GUARD(s2n_stuffer_write_reservation(&available_cipher_suites_size, ciphers_size));
 
     /* Zero compression methods */
     POSIX_GUARD(s2n_stuffer_write_uint8(out, 1));
@@ -821,7 +824,7 @@ int s2n_sslv2_client_hello_recv(struct s2n_connection *conn)
     POSIX_GUARD(s2n_stuffer_skip_write(&in_stuffer, client_hello->raw_message.size));
     struct s2n_stuffer *in = &in_stuffer;
 
-    const struct s2n_security_policy *security_policy;
+    const struct s2n_security_policy *security_policy = NULL;
     POSIX_GUARD(s2n_connection_get_security_policy(conn, &security_policy));
 
     if (conn->client_protocol_version < security_policy->minimum_protocol_version) {
@@ -831,15 +834,15 @@ int s2n_sslv2_client_hello_recv(struct s2n_connection *conn)
     conn->actual_protocol_version = MIN(conn->client_protocol_version, conn->server_protocol_version);
 
     /* We start 5 bytes into the record */
-    uint16_t cipher_suites_length;
+    uint16_t cipher_suites_length = 0;
     POSIX_GUARD(s2n_stuffer_read_uint16(in, &cipher_suites_length));
     POSIX_ENSURE(cipher_suites_length > 0, S2N_ERR_BAD_MESSAGE);
     POSIX_ENSURE(cipher_suites_length % S2N_SSLv2_CIPHER_SUITE_LEN == 0, S2N_ERR_BAD_MESSAGE);
 
-    uint16_t session_id_length;
+    uint16_t session_id_length = 0;
     POSIX_GUARD(s2n_stuffer_read_uint16(in, &session_id_length));
 
-    uint16_t challenge_length;
+    uint16_t challenge_length = 0;
     POSIX_GUARD(s2n_stuffer_read_uint16(in, &challenge_length));
 
     S2N_ERROR_IF(challenge_length > S2N_TLS_RANDOM_DATA_LEN, S2N_ERR_BAD_MESSAGE);
@@ -879,7 +882,7 @@ int s2n_client_hello_get_parsed_extension(s2n_tls_extension_type extension_type,
     POSIX_ENSURE_REF(parsed_extension_list);
     POSIX_ENSURE_REF(parsed_extension);
 
-    s2n_extension_type_id extension_type_id;
+    s2n_extension_type_id extension_type_id = 0;
     POSIX_GUARD(s2n_extension_supported_iana_value_to_id(extension_type, &extension_type_id));
 
     s2n_parsed_extension *found_parsed_extension = &parsed_extension_list->parsed_extensions[extension_type_id];
